@@ -40,11 +40,35 @@ def getdefint(sct,key,val):
     except:
         return val
 
+Achievements=[  ['Not even trying','you never even tried! *SOB*',True,0], 
+                ['Quack Quack', 'you scored a whopping zero, your parents must be very proud',True,0],
+                ['OK','Max thinks you are OK, me, I am not so sure', True,0],
+                ['GOOD','Max thinks you are GOOD, he could have a point', True,0],
+                ['EXCELLENT', 'Max knows you are Excellent', True,0],
+                ['FANTASTIC', 'No really, jolly good show, well done', True,0],
+                ['Rock around the clock', 'No really, jolly good show, well done', True,0],
+                ['Tooth fairy', 'now you are 5, expect a present under your pillow tonight',True,0],
+                ['7-up','personally I prefer sprite',True,0],
+                ['Teenage dirtbag', 'you should probably swap playing for alcopops and a park bench',True,0],
+                ['Sweet sixteen','sweeeeeeeet',True,0],
+                ['Nuh-nuh-nineteen', 'NINETEEN!!! NINETEEN',True,0],
+                ['999', "scarper it's the fuzz", True,0],
+                ['Hurry hurry hurry', 'are you EVER on time???',True,0],
+                ['Perfect 10', "scarper it's the fuzz", True,0],
+                ['Three touch', "", True,0],
+                ['Two touch', "", True,0],
+                ['All growed up','21 already.. I think you have outgrown achievements', True,0]]
+
+
 def saveconfig():
     configfile = open(settingsdir +'/issettings.cfg', "w")
     print "writting settings"
     config.write(configfile)
 
+def setachievements():
+    for i in range(len(Achievements)):
+        config.set('achievements',Achievements[i][0], Achievements[i][3])
+
 try:
     configfile = open(settingsdir +'/issettings.cfg', "r")
     config.readfp(configfile)
@@ -53,6 +77,8 @@ try:
     showfps=getdefint('settings', 'showfps',0)==1
     startlevel=getdefint('settings', 'startlevel',0)
     fullscreenrefresh=getdefint('settings', 'fullscreenrefresh',0)==1
+    for i in range(len(Achievements)):
+        Achievements[i][3] =getdefint('achievements', Achievements[i][0],0)
 except:
     print configfilename + " - read failure, picking defaults"
     hiscore = 2000
@@ -126,9 +152,18 @@ outerwidth = 220
 
 gcsnum=0
 
+def addachievement(achname):
+    for i in range(len(Achievements)):
+        if Achievements[i][0] == achname:
+            Achievements[i][3] = Achievements[i][3] +1
+
 pauseIconTop=202
 soundIconTop=390
 
+achieveIconTop=204
+achieveIconLeft=20
+
+
 InfoIconTop=390
 InfoIconLeft=20
 
@@ -176,7 +211,7 @@ def transparentise(surf,maxalpha):
     for x in range(width):
         for y in range(height):
             red, green, blue, alpha = surf.get_at((x, y))
-            L = 0.3 * red + 0.59 * green + 0.11 * blue
+            L = 0.3 * red + 0.59 * pgreen + 0.11 * blue
             if alpha > maxalpha:
                 alpha = maxalpha
             gs_color = (red, green, blue, alpha)
@@ -194,6 +229,8 @@ def renderbuttons(background):
     global infoicongs
     global pauseicon
     global pauseicongs
+    global achieveicon
+    global achieveicongs
     global xicon
     global xicongs
 
@@ -218,6 +255,11 @@ def renderbuttons(background):
         infoicongs = pygame.image.load(appdir + "/Icons/info.png")
         infoicongs = grayscale_image(infoicongs)
 
+        achieveicon = pygame.image.load(appdir + "/Icons/star.png")
+        achieveicongs = pygame.image.load(appdir + "/Icons/star.png")
+        achieveicongs = grayscale_image(achieveicongs)
+
+
     if playsounds:
         background.blit(soundicon, (724,soundIconTop))
     else:
@@ -225,6 +267,8 @@ def renderbuttons(background):
 
     background.blit(infoicongs, (InfoIconLeft,InfoIconTop))
 
+    background.blit(achieveicongs, (achieveIconLeft,achieveIconTop))
+
     background.blit(pauseicongs, (720,pauseIconTop))
     background.blit(xicongs, pos_close)
 
@@ -314,13 +358,9 @@ def handlebuttonpress(event,screen,background):
             soundplay(soundonsnd)
             config.set('settings','sound', '1')
             saveconfig()
-#            configfile.seek(0)
-#            config.write(configfile)
         else:
             config.set('settings','sound', '0')
             saveconfig()
-#            configfile.seek(0)
-#            config.write(configfile)
 
         if playsounds:
             oggplayer.set_state(gst.STATE_PLAYING)
@@ -334,6 +374,10 @@ def handlebuttonpress(event,screen,background):
     elif (event.pos[0]<80) and (event.pos[1])>InfoIconTop and (event.pos[1])<InfoIconTop+64:
         showInfo(screen,background)
         return True
+    elif (event.pos[0]<80) and (event.pos[1])>achieveIconTop and (event.pos[1])<achieveIconTop+64:
+        showAchievements(screen,background)
+        return True
+
     else:
         return False
 
@@ -401,6 +445,46 @@ def Betweengames(score, screen):
     textRect = text.get_rect(centerx=400,centery=350)
     background.blit(text,textRect)
 
+ 
+
+    if score == 0:
+        addachievement('Quack Quack')
+
+    if score >= 0:
+        if score == 0:
+            infosnd=pygame.mixer.Sound(appdir + "/sounds/quackquack.wav")
+        elif score < 800:   
+            infosnd=pygame.mixer.Sound(appdir + "/sounds/badscore.wav")
+        elif score < 1600:   
+            infosnd=pygame.mixer.Sound(appdir + "/sounds/okscore.wav")
+            addachievement('OK')
+        elif score < 3200:   
+            infosnd=pygame.mixer.Sound(appdir + "/sounds/goodscore.wav")
+            addachievement('GOOD')
+        elif score < 6400:   
+            infosnd=pygame.mixer.Sound(appdir + "/sounds/excellentscore.wav")
+            addachievement('EXCELLENT')
+        else:
+            infosnd=pygame.mixer.Sound(appdir + "/sounds/fantasticscore.wav")
+            addachievement('FANTASTIC')
+
+        if score == hiscore:
+            infosnd=pygame.mixer.Sound(appdir + "/sounds/highscore.wav")
+
+
+
+    font = pygame.font.Font(None, 32,bold=True)
+    countAchievements=0
+    for i in range(len(Achievements)):
+        if Achievements[i][3] > 0:
+            countAchievements = countAchievements +1
+
+    text = font.render('Achievements ('+str(countAchievements) + '/' + str(len(Achievements)) +')', True, (255,255, 255), (0,64,0))
+    # Create a rectangle
+    text.set_colorkey((0,64,0))
+    textRect = text.get_rect(centerx=400,centery=440)
+    background.blit(text,textRect)
+
 #    pygame.display.flip()
     screen.blit(background,(0,0))
     clock = pygame.time.Clock()
@@ -414,26 +498,14 @@ def Betweengames(score, screen):
         pygame.display.update([rect])
 
 
-    if score >= 0:
-        if score == hiscore:
-            infosnd=pygame.mixer.Sound(appdir + "/sounds/highscore.wav")
-        elif score < 800:   
-            infosnd=pygame.mixer.Sound(appdir + "/sounds/badscore.wav")
-        elif score < 1600:   
-            infosnd=pygame.mixer.Sound(appdir + "/sounds/okscore.wav")
-        elif score < 3200:   
-            infosnd=pygame.mixer.Sound(appdir + "/sounds/goodscore.wav")
-        elif score < 6400:   
-            infosnd=pygame.mixer.Sound(appdir + "/sounds/excellentscore.wav")
-        else:
-            infosnd=pygame.mixer.Sound(appdir + "/sounds/fantasticscore.wav")
+    if score >= 0: 
         soundplay(infosnd)
 
     if score == hiscore:
         config.set('achievements','highscore', str(score))
-        saveconfig()
-#        configfile.seek(0)
-#        config.write(configfile)
+
+    setachievements()        
+    saveconfig()
 
     mdown=False
     while 1:
@@ -442,12 +514,12 @@ def Betweengames(score, screen):
             return True
 
         clock.tick(10)
-#       pretty hideous way of detcting song end... but seems to work
+#       pretty hideous way of detecting EOS mp3... but seems to work
         message = bus.poll(gst.MESSAGE_EOS,0)
         if message != None:
+            print "EOS message popped (leaving between game state)"
             return True
 
-
         for event in pygame.event.get():
             if event.type == QUIT:
                 return False
@@ -459,6 +531,9 @@ def Betweengames(score, screen):
             if event.type == MOUSEBUTTONUP:
                 if mdown:
                     return True
+            if event.type == ACTIVEEVENT:
+                print "Deactivated"
+                pause(screen, background, True)                        
 
 
 def anglediff(a1, a2):
@@ -490,14 +565,116 @@ def calcspritetl(angle, distance):
     y = 224+(math.sin(rang) * distance)
     return (x,y)
 
+
+def showAchievements(screen,background):
+    oggplayer.set_state(gst.STATE_PAUSED)
+    infosnd=pygame.mixer.Sound(appdir + "/sounds/achievements.wav")
+    soundplay(infosnd)
+
+    savebackground =pygame.Surface((800,480))
+    savebackground.blit(screen,(0,0,800,480))
+    screen.blit(achieveicon,(achieveIconLeft,achieveIconTop))
+    screen.blit(achieveicon,(68,0))
+    screen.blit(achieveicon,(668,0))
+    screen.blit(achieveicon,(68,416))
+    screen.blit(achieveicon,(668,416))
+
+
+    pygame.display.flip()
+
+    textsurf = pygame.Surface((600,440))
+    clrinfo=[255,200,200]
+    textsurf.fill(clrinfo)
+    pygame.draw.rect(textsurf,[0,0,127],(0,0,600,440),5)
+
+    font = pygame.font.Font(None, 34)
+    headfont = pygame.font.Font(None, 48)
+
+    text = headfont.render('ACHIEVEMENTS', True, (0, 0, 127), clrinfo)
+    textRect = text.get_rect(centerx=300,centery=30)
+    textsurf.blit(text,textRect)
+
+    for i in range(11):
+        if Achievements[i][3] > 0:
+            text = font.render(Achievements[i][0] + ': ' +Achievements[i][1], True, (0, 0, 127), clrinfo)
+            text = font.render(Achievements[i][0], True, (0, 0, 127), clrinfo)
+        else: 
+            text = font.render(Achievements[i][0], True, (127, 127, 127), clrinfo)     
+        textRect = text.get_rect(x=30,centery=84 +(i*32))
+        textsurf.blit(text,textRect)
+
+    for i in range(11,len(Achievements)):
+        if Achievements[i][3] > 0:
+            text = font.render(Achievements[i][0] + ': ' +Achievements[i][1], True, (0, 0, 127), clrinfo)
+            text = font.render(Achievements[i][0], True, (0, 0, 127), clrinfo)
+        else: 
+            text = font.render(Achievements[i][0], True, (127, 127, 127), clrinfo)     
+        textRect = text.get_rect(x=330,centery=84 +((i-11)*32))
+        textsurf.blit(text,textRect)
+
+
+    textsurf.set_alpha(225)
+
+    screen.blit(textsurf,(100,20))       
+
+    screen.blit(achieveicon,(68,0))
+    screen.blit(achieveicon,(668,0))
+    screen.blit(achieveicon,(68,416))
+    screen.blit(achieveicon,(668,416))
+
+    rect= Rect((0,480),(800,8))
+    ypos = 480
+    clock = pygame.time.Clock()
+    while rect.top > -8:
+        clock.tick(60)
+        rect.centery=ypos
+        ypos=ypos-8
+        pygame.display.update([rect])
+
+    clock = pygame.time.Clock()
+    mdown=False
+    while 1:
+        clock.tick(10)
+        for event in pygame.event.get():
+            if event.type == QUIT:
+                return False
+            if event.type == MOUSEBUTTONDOWN:
+                mdown=True
+            if event.type == MOUSEBUTTONUP:
+                if mdown:
+                    if playsounds:
+                        oggplayer.set_state(gst.STATE_PLAYING)
+                    screen.blit(savebackground, (0,0,800,480))  
+                    pygame.display.flip()
+                    return True
+
+
 def pause(screen,background,silent):
     oggplayer.set_state(gst.STATE_PAUSED)
-
-    infosnd=pygame.mixer.Sound(appdir + "/sounds/pause.wav")
-    soundplay(infosnd)
+    if not silent:
+        infosnd=pygame.mixer.Sound(appdir + "/sounds/pause.wav")
+        soundplay(infosnd)
     savebackground =pygame.Surface((800,480))
     savebackground.blit(screen,(0,0,800,480))
     screen.blit(pauseicon,(720,pauseIconTop))
+
+    textsurf = pygame.Surface((600,440))
+    clrinfo=[255,200,200]
+    textsurf.fill(clrinfo)
+    pygame.draw.rect(textsurf,[0,0,127],(0,0,600,440),5)
+
+    font = pygame.font.Font(None, 38)
+    headfont = pygame.font.Font(None, 192)
+
+    text = headfont.render("PAUSE", True, (0, 0, 127), clrinfo)
+    textRect = text.get_rect(centerx=300,centery=220)
+    textsurf.blit(text,textRect)
+
+    textsurf.set_alpha(225)
+
+    screen.blit(textsurf,(100,20))       
+
+
     pygame.display.flip()
     clock = pygame.time.Clock()
     mdown=False
@@ -601,6 +778,9 @@ def showInfo(screen,background):
                     screen.blit(savebackground, (0,0,800,480))  
                     pygame.display.flip()
                     return True
+            if event.type == ACTIVEEVENT:
+                print "Deactivated"
+                pause(screen, background, True)                        
 
 def PlayGame(screen):
     oggplayer.set_state(gst.STATE_NULL)
@@ -623,6 +803,11 @@ def PlayGame(screen):
     cdfont = pygame.font.Font(None, 100)
     text = font.render('', True, (255,255, 255), (0,64,0))
 
+    hurrycount=0
+    ninecount=0
+    levelcompletions=[0,0,0,0,0,0,0,0,0,0]
+    touches=0
+
     # Create a rectangle
     textRect = text.get_rect()
     timetextRect = textRect
@@ -676,6 +861,7 @@ def PlayGame(screen):
         if message != None:
             print "EOS message popped (rewinding)"
             oggplayer.seek_simple(gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH, 0)
+            addachievement('Rock around the clock')
 
         gameticks = clock.tick(60)*60/1000
     
@@ -695,11 +881,15 @@ def PlayGame(screen):
         for event in pygame.event.get():
             if event.type == QUIT:
                 return False
+            if event.type == ACTIVEEVENT:
+                print "Deactivatd"
+                pause(screen, background, True)                        
             if event.type == MOUSEBUTTONDOWN:
                 if handlebuttonpress(event,screen,background):
                     pass
                 else:
                     mdown = True
+                    touches = touches +1
                     dist = math.sqrt((event.pos[0] - centerpoint[0])**2
                          + (event.pos[1] - centerpoint[1])**2)
 
@@ -810,6 +1000,7 @@ def PlayGame(screen):
             rectlist.append(countdowntextRect)
             if (int(timeremaining)<=500) and not playedhurry:
                 soundplay(hurrysnd)
+                hurrycount=hurrycount +1
                 playedhurry=True
 
             text = cdfont.render(str(int(timeremaining/200) +1),
@@ -861,7 +1052,6 @@ def PlayGame(screen):
         for bs in level.outersprites:
             alllit = alllit and bs.lit
 
-
 # play sounds depending on state changes
         playon=False
         playoff=False
@@ -922,6 +1112,18 @@ def PlayGame(screen):
         
         if alllit:
             soundplay(lvldonesnd)
+
+            if playedhurry and (hurrycount == 3):
+                addachievement('Hurry hurry hurry')
+
+            if lastcount == 10:
+                addachievement('Perfect 10')
+
+            if lastcount == 9:
+                ninecount = ninecount +1
+                if (ninecount ==3):
+                    addachievement('999')
+
             playedhurry=False
             effectsprites=[]
             for bs in level.outersprites:
@@ -930,6 +1132,13 @@ def PlayGame(screen):
                 effectsprites.append(bs)
             haseffects=True
 
+            if touches <= 2:
+                addachievement('Two touch')
+            elif touches == 3:
+                addachievement('Three touch')
+
+            touches =0
+
             background = setupbackground()
             score=score + ((int(timeremaining/20) +1) * (levelNo +1))
             print "Completed Level: " + str(levelNo +1)
@@ -937,6 +1146,20 @@ def PlayGame(screen):
             print "score: " + str(score)
             print "New Level"
             levelNo = levelNo + 1
+
+            if levelNo == 5:
+                addachievement('Tooth fairy')
+            if levelNo == 5:
+                addachievement('7-up')
+            if levelNo == 13:
+                addachievement('Teenage dirtbag')
+            if levelNo == 16:
+                addachievement('Sweet sixteen')
+            if levelNo == 19:
+                addachievement('Nuh-nuh-nineteen')
+            if levelNo == 21:
+                addachievement('All growed up')
+
             # set mouse up (even if untrue)
             mdown=False
             # clear what went before...
@@ -967,15 +1190,15 @@ def PlayGame(screen):
     text = cdfont.render("TIME'S UP!",
            True, (255,127,127), (0,64,0))
 
+    if (score == 0) and (touches ==0):
+        addachievement('Not even trying')
+
     text.set_colorkey((0,64,0))
 
-
-    # Create a rectangle
     countdowntextRect = text.get_rect( centerx=400, centery=240)
     screen.blit(text,countdowntextRect)
     rectlist.append(countdowntextRect)
     pygame.display.update(rectlist)
-#    oggplayer.disconnect(repeatid)
     return score
 
 
